#version 330                                                                        
                                                                                    
layout(points) in;                                                                  
layout(points) out;                                                                 
layout(max_vertices = 100) out;                                                      
                                                                                    
in float Type0[];                                                                   
in vec3 Position0[];                                                                
in vec3 Velocity0[];                                                                
in float Age0[];                                                                    
                                                                                    
out float Type1;                                                                    
out vec3 Position1;                                                                 
out vec3 Velocity1;                                                                 
out float Age1;                                                                     
                                                                                    
uniform float gDeltaTimeMillis;                                                     
uniform float gTime;                                                                
uniform sampler1D gRandomTexture;                                                   
uniform float gLauncherLifetime;                                                    
uniform float gShellLifetime;                                                       
uniform float gSecondaryShellLifetime;
uniform vec3 spawnPosition;
uniform vec3 spawnDirection;       
uniform float explosion;                                      
                                                                                    
#define PARTICLE_TYPE_LAUNCHER 0.0f                
#define PARTICLE_TYPE_THRUSTER 3.0f                                         
#define PARTICLE_TYPE_SHELL 1.0f                                                    
#define PARTICLE_TYPE_SECONDARY_SHELL 2.0f          
#define PARTICLE_TYPE_EXPLOSION 4.0f         
#define PARTICLE_TYPE_EXPLOSION_SECONDARY 5.0f     
#define PARTICLE_TYPE_FIRE 6.0f                              
                                                                                    
vec3 GetRandomDir(float TexCoord)                                                   
{                                                                                   
     vec3 Dir = texture(gRandomTexture, TexCoord).xyz;                              
     Dir -= vec3(0.5, 0.5, 0.5);                                                    
     return Dir;                                                                    
}                                                                                   
                                                                                    
void main()                                                                         
{                                                                                   
    float Age = Age0[0] + gDeltaTimeMillis;                                         
                                                                                    
    if (Type0[0] == PARTICLE_TYPE_LAUNCHER) {                                       
        if (Age >= gLauncherLifetime && gTime > 83000){                                             
            Type1 = PARTICLE_TYPE_SHELL;                                            
            Position1 = Position0[0];                                                                            
            Velocity1 = (Velocity0[0] + GetRandomDir(gTime/3000))*3;                                      
            Age1 = 0.0;                                                             
            EmitVertex();                                                           
            EndPrimitive();                                                         
            Age = 0.0;                                                              
        }                                                                           
                                                                                    
        Type1 = PARTICLE_TYPE_LAUNCHER;                                             
        Position1 = spawnPosition;                                                   
        Velocity1 = spawnDirection;                                                 
        Age1 = Age;                                                                 
        EmitVertex();                                                               
        EndPrimitive();                                                             
    }     
	else if (Type0[0] == PARTICLE_TYPE_THRUSTER) {                                       
        if (Age >= gLauncherLifetime && gTime > 55000) {     
			for(int i = 1; i < 15; i++)
			{                                        
				Type1 = PARTICLE_TYPE_FIRE;                                            
				Position1 = Position0[0];                                               
				vec3 Dir = GetRandomDir((gTime*i)/5000.0);                                                                               
				Velocity1 = Velocity0[0]*2 + normalize(Dir) ;                                
				Age1 = 0.0;                                                             
				EmitVertex();                                                           
				EndPrimitive();                                                         
				Age = 0.0;                                                              
			}
        }                                                                           
                                                                                    
        Type1 = PARTICLE_TYPE_THRUSTER;                                             
        Position1 = spawnPosition;                                                   
        Velocity1 = spawnDirection;                                                   
        Age1 = Age;                                                                 
        EmitVertex();                                                               
        EndPrimitive();                                                             
    }       
	else if (Type0[0] == PARTICLE_TYPE_EXPLOSION)
	{
	    if (Age0[0] >= 65900 ) {     
			for(int i = 0; i < 80; i++)
			{                                        
				Type1 = PARTICLE_TYPE_EXPLOSION_SECONDARY;                                            
				Position1 = Position0[0];                                               
				Velocity1 = Velocity0[0] + GetRandomDir((gTime*i)/1000.0)*20;                                      
				Age1 = Age;                                                             
				EmitVertex();                                                           
				EndPrimitive();                                                               
			}
        }                                                               
                                                                                    
        Type1 = PARTICLE_TYPE_EXPLOSION;                                             
        Position1 = spawnPosition;                                                   
        Velocity1 = spawnDirection;                                                   
        Age1 = Age;                                                                 
        EmitVertex();                                                               
        EndPrimitive(); 
		
	} else if (Type0[0] == PARTICLE_TYPE_EXPLOSION_SECONDARY)
	{
			for(int i = 0; i < 10; i++)
			{                                        
				Type1 = PARTICLE_TYPE_FIRE;                                            
				Position1 = Position0[0];                                               
				Velocity1 = Velocity0[0] +  GetRandomDir((gTime*i)/1000.0)*20;                                      
				Age1 = 0.0;                                                             
				EmitVertex();                                                           
				EndPrimitive();                                                         
				Age = 0.0;                                                              
			}                                                                       

	}                                                                        
    else 
	{                                                                          
        float DeltaTimeSecs = gDeltaTimeMillis / 1000.0f;                           
        float t1 = Age0[0] / 1000.0;                                                
        float t2 = Age / 1000.0;                                                    
        vec3 DeltaP = DeltaTimeSecs * Velocity0[0];                                 
        vec3 DeltaV = vec3(DeltaTimeSecs) * (0.0, 10, 0.0);                      
                                                                                    
		if (Type0[0] == PARTICLE_TYPE_SHELL)  
			{                                     
				if (Age < gShellLifetime) {                                             
					Type1 = PARTICLE_TYPE_SHELL;                                        
					Position1 = Position0[0] + DeltaP;                                  
					Velocity1 = Velocity0[0];                                  
					Age1 = Age;                                                         
					EmitVertex();                                                       
					EndPrimitive();                                                     
			 }      
        else if (Type0[0] == PARTICLE_TYPE_FIRE)  
		{                                     
				if (Age < gShellLifetime)
				{                                              
					Type1 = PARTICLE_TYPE_FIRE;                                        
					Position1 = Position0[0] + DeltaP;                                  
					Velocity1 = Velocity0[0];                                  
					Age1 = Age;                                                         
					EmitVertex();                                                       
					EndPrimitive();                                                     
				} 
		}  else   
			 {                                                                  
			 for (int i = 0 ; i < 25 ; i++)
			 {                                    
						 Type1 = PARTICLE_TYPE_SECONDARY_SHELL;                         
						 Position1 = Position0[0];                                      
						 vec3 Dir = GetRandomDir((gTime*i)/3000.0);                   
						 Velocity1 = Dir ;                             
						 Age1 = 0.0f;                                                   
						 EmitVertex();                                                  
						 EndPrimitive();                                                
			 }                                                                   
        }
		                                                                                                     
                                                    
       }  
	  else {                                                                      
            if (Age < gSecondaryShellLifetime) {                                    
                Type1 = PARTICLE_TYPE_SECONDARY_SHELL;                              
                Position1 = Position0[0] + DeltaP;                                  
                Velocity1 = Velocity0[0] + DeltaV;                                  
                Age1 = Age;                                                         
                EmitVertex();                                                       
                EndPrimitive();                                                     
            }                                                                       
        }                                
                                     
     } 
                                                                                                                  
}                                                                                   
