#version 330

in vec2 TexCoord0;
in vec3 NormalW;
in vec3 Tangent0;

out vec4 FragColor;


struct DiffuseLight                                                    
{                                                                                   
    vec3 color;                                                                                                                           
    float diffuseIntensity;                                                         
    vec3 direction;                                                                 
};


uniform sampler2D gSampler;
uniform DiffuseLight diffuseLight;
uniform float ambientIntensity;
uniform sampler2D gNormalMap;  



vec3 CalcBumpedNormal()
{
    vec3 Normal = normalize(NormalW);
    vec3 Tangent = normalize(Tangent0);
    Tangent = normalize(Tangent - dot(Tangent, Normal) * Normal);
    vec3 Bitangent = cross(Tangent, Normal);
    vec3 BumpMapNormal = texture(gNormalMap, TexCoord0).xyz;
    BumpMapNormal = 2.0 * BumpMapNormal - vec3(1.0, 1.0, 1.0);
    vec3 NewNormal;
    mat3 TBN = mat3(Tangent, Bitangent, Normal);
    NewNormal = TBN * BumpMapNormal;
    NewNormal = normalize(NewNormal);
    return NewNormal;
}


void main()
{
    vec4 ambientColor = vec4(0.5f,0.5f,0.5f,1.0f) * ambientIntensity;
    vec3 Normal = CalcBumpedNormal();
   
   float diffuseFactor = dot(Normal, -diffuseLight.direction);

   vec4 diffuseColor;
   diffuseColor = vec4(diffuseLight.color, 1.0f) *
                        diffuseLight.diffuseIntensity *
                        diffuseFactor;

    FragColor = texture2D(gSampler, TexCoord0.xy) * diffuseColor + ambientColor ;

}